@@ -267,25 +267,78 @@ def _para_intersection_poset(A):
     return [Poset((t, cmp_fn)), label_dict, hyp_dict]
 
 
-def _lof_from_matroid(A):
+# Default SageMath algorithm works well. However 'A.matroid()' seems to remove
+# ordering, which we depend on, so care is needed. 
+def _lof_from_matroid(A=None, matroid=None):
     from sage.all import Set, Matrix, Matroid, Poset
     from functools import reduce
-    rows = list(map(lambda H: H.coefficients()[1:], A.hyperplanes()))
-    mat = Matrix(A.base_ring(), rows).transpose()
-    M = Matroid(mat)
+    if A != None:
+        rows = list(map(lambda H: H.coefficients()[1:], A.hyperplanes()))
+        mat = Matrix(A.base_ring(), rows).transpose()
+        M = Matroid(mat)
+        n = len(A)
+        lbl_map = lambda S: S 
+    else:
+        M = matroid.simplify()
+        n = len(M.groundset())
+        grd_list = list(M.groundset())
+        l_map = {x : grd_list.index(x) for x in grd_list}
+        lbl_map = lambda S: frozenset([l_map[x] for x in S])
+
     L = M.lattice_of_flats()
-    ranks = reduce(lambda x, y: x+y, [list(filter(lambda x: L.rank(x) == r, L._elements)) for r in range(2, L.rank() + 1)], [L.bottom()] + [frozenset([k]) for k in range(len(A))])
-    P = Poset(L, element_labels={x : ranks.index(x) for x in L._elements})
+    rank_r = lambda L, r: list(
+        map(lbl_map, filter(lambda x: L.rank(x) == r, L._elements))
+    )
+    rank_1 = [frozenset([k]) for k in range(n)]
+    ranks = reduce(
+        lambda x, y: x + y, 
+        [rank_r(L, r) for r in range(2, L.rank() + 1)], 
+        [L.bottom()] + rank_1
+    )
+    P = Poset(
+        L, element_labels={x : ranks.index(lbl_map(x)) for x in L._elements}
+    )
     adj_set = lambda S: Set([x+1 for x in S])
     label_dict = {i : adj_set(ranks[i]) for i in range(len(L))}
-    hyp_dict = {i : A[list(ranks[i])[0]] for i in range(1, len(A) + 1)}
+    if A != None:
+        hyp_dict = {i : A[list(ranks[i])[0]] for i in range(1, n + 1)}
+    else: 
+        hyp_dict = None 
     return [P, label_dict, hyp_dict]
 
 
+def _lof_from_affine_matroid(A):
+    from sage.all import Poset, Set 
+    from functools import reduce
+    A_coned = A.cone()
+    hyps = list(map(lambda H: H.coefficients(), A_coned.hyperplanes()))
+    extra = [0, 1] + [0]*(A.dimension())
+    i = hyps.index(extra)
+    P, L, H = _lof_from_matroid(A_coned)
+    assert (H[i + 1]).coefficients() == extra 
+    new_elts = list(filter(lambda x: not P.le(i + 1, x), P))
+    new_elts = reduce(
+        lambda x, y: x + y,
+        [list(filter(lambda x: P.rank(x) == r, new_elts)) for r in range(2, P.rank() + 1)],
+        [0] + [j for j in range(1, len(A_coned) + 1) if j != i + 1]
+    )
+    new_names = {x : new_elts.index(x) for x in new_elts}
+    adj_set = lambda S: Set([new_names[x] for x in S if x in new_elts])
+    P_new = Poset(P.subposet(new_elts), element_labels=new_names)
+    L_new = {new_names[x] : adj_set(L[x]) for x in new_elts}
+    def inv_H(h): 
+        cut = lambda x: x.coefficients()[1:]
+        pair = list(filter(lambda x: cut(x[1]) == h, H.items())) 
+        return pair[0][0]
+    H_new = {new_names[inv_H(h.coefficients())] : h for h in A}
+    return [P_new, L_new, H_new]
+
+
 class LatticeOfFlats():
 
     def __init__(self, A=None, poset=None, flat_labels=None, 
-    hyperplane_labels=None, lazy=False, nature_hyperplane_label=True):
+    hyperplane_labels=None, lazy=False, matroid=None, 
+    nature_hyperplane_label=True):
         self.hyperplane_arrangement = A
         self.poset = poset 
         self.flat_labels = flat_labels
@@ -296,10 +349,13 @@ class LatticeOfFlats():
             self.poset = poset
         else:
             if not lazy:
-                if A.is_central():
-                    P, FL, HL = _lof_from_matroid(A)
-                else:
-                    P, FL, HL = _para_intersection_poset(A)
+                if A != None: 
+                    if A.is_central():
+                        P, FL, HL = _lof_from_matroid(A)
+                    else:
+                        P, FL, HL = _lof_from_affine_matroid(A)
+                else: 
+                    P, FL, HL = _lof_from_matroid(A=None, matroid=matroid)
                 self.poset = P
                 self.flat_labels = FL
                 self.hyperplane_labels = HL
@@ -498,32 +554,32 @@ class LatticeOfFlats():
 
     @cached_method
     def Poincare_polynomial(self):
-        from sage.all import QQ
-        from sage.rings.polynomial.polynomial_ring import polygen
-        Y = polygen(QQ, 'Y')
+        from sage.all import QQ, PolynomialRing
+        PR = PolynomialRing(QQ, 'Y')
+        Y = PR.gens()[0]
         if self.poset != None:
             P = self.poset 
             atoms = self.atoms()
             if P.rank() == 0:
-                return QQ(1)
+                return PR(1)
             if P.rank() == 1:
-                return QQ(1) + len(atoms)*Y
+                return PR(1 + len(atoms)*Y)
         else: 
             # Lazy 
             A = self.hyperplane_arrangement
             assert A != None, "Expected either a poset or hyperplane arrangement."
             if A.rank() == 0:
-                return QQ(1)
+                return PR(1)
             if A.rank() == 1:
-                return QQ(1) + len(A)*Y
+                return PR(1 + len(A)*Y)
         if self.hyperplane_arrangement == None:
             chi = self.poset.characteristic_polynomial()
             q = chi.variables()[0]
             d = chi.degree(q)
-            return (-Y)**d*chi.subs({q : -Y**-1})
+            return PR((-Y)**d*chi.subs({q : -Y**-1}))
         D = self._lazy_deletion(1)
         R = self._lazy_restriction(1)
-        return D.Poincare_polynomial() + Y*R.Poincare_polynomial()
+        return PR(D.Poincare_polynomial() + Y*R.Poincare_polynomial())
         
     @cached_method
     def _combinatorial_eq_elts(self):
